@@ -24,7 +24,6 @@ configuration on user login.
 @click.option("--no", "-N", is_flag=True, default=False)
 @existing_config_option
 def autostart(yes: bool, no: bool, config_name: str) -> None:
-
     from ..autostart import AutoStart
 
     auto_start = AutoStart(config_name)
@@ -51,14 +50,13 @@ def autostart(yes: bool, no: bool, config_name: str) -> None:
 
 
 @click.group(help="View and manage excluded folders.")
-def excluded():
+def excluded() -> None:
     pass
 
 
 @excluded.command(name="list", help="List all excluded files and folders.")
 @inject_proxy(fallback=True, existing_config=True)
 def excluded_list(m: Maestral) -> None:
-
     excluded_items = m.excluded_items
     excluded_items.sort()
 
@@ -77,7 +75,6 @@ def excluded_list(m: Maestral) -> None:
 @inject_proxy(fallback=True, existing_config=True)
 @convert_api_errors
 def excluded_add(m: Maestral, dropbox_path: str) -> None:
-
     if dropbox_path == "/":
         raise CliException("Cannot exclude the root directory.")
 
@@ -99,7 +96,6 @@ folder will be included as well (but no other items inside it).
 @inject_proxy(fallback=False, existing_config=True)
 @convert_api_errors
 def excluded_remove(m: Maestral, dropbox_path: str) -> None:
-
     if dropbox_path == "/":
         return echo("The root directory is always included")
 
@@ -108,7 +104,7 @@ def excluded_remove(m: Maestral, dropbox_path: str) -> None:
 
 
 @click.group(help="Manage desktop notifications.")
-def notify():
+def notify() -> None:
     pass
 
 
@@ -123,7 +119,6 @@ def notify():
 )
 @inject_proxy(fallback=True, existing_config=True)
 def notify_level(m: Maestral, level_name: str) -> None:
-
     from .. import notify as _notify
 
     if level_name:
@@ -147,3 +142,47 @@ def notify_snooze(m: Maestral, minutes: int) -> None:
         ok(f"Notifications snoozed for {minutes} min. Set snooze to 0 to reset.")
     else:
         ok("Notifications enabled.")
+
+
+@click.group(help="View and manage bandwidth limits. Changes take effect immediately.")
+def bandwidth_limit() -> None:
+    pass
+
+
+@bandwidth_limit.command(
+    name="up", help="Get / set bandwidth limit for uploads in MB/sec (0 = unlimited)."
+)
+@click.argument(
+    "mb_per_second",
+    required=False,
+    type=click.FLOAT,
+)
+@inject_proxy(fallback=True, existing_config=True)
+def bandwidth_limit_up(m: Maestral, mb_per_second: float | None) -> None:
+    if mb_per_second is not None:
+        m.bandwidth_limit_up = mb_per_second * 10**6
+        speed_str = f"{mb_per_second} MB/sec" if mb_per_second > 0 else "unlimited"
+        ok(f"Upload bandwidth limit set to {speed_str}.")
+    else:
+        mb_per_second = m.bandwidth_limit_up / 10**6
+        echo(f"{mb_per_second} MB/sec" if mb_per_second > 0 else "unlimited")
+
+
+@bandwidth_limit.command(
+    name="down",
+    help="Get / set bandwidth limit for downloads in MB/sec (0 = unlimited).",
+)
+@click.argument(
+    "mb_per_second",
+    required=False,
+    type=click.FLOAT,
+)
+@inject_proxy(fallback=True, existing_config=True)
+def bandwidth_limit_down(m: Maestral, mb_per_second: float | None) -> None:
+    if mb_per_second is not None:
+        m.bandwidth_limit_down = mb_per_second * 10**6
+        speed_fmt = f"{mb_per_second} MB/sec" if mb_per_second > 0 else "unlimited"
+        ok(f"Download bandwidth limit set to {speed_fmt}.")
+    else:
+        mb_per_second = m.bandwidth_limit_down / 10**6
+        echo(f"{mb_per_second} MB/sec" if mb_per_second > 0 else "unlimited")
