@@ -7,7 +7,7 @@ from __future__ import annotations
 
 import os
 from os import path as osp
-from typing import IO
+from typing import Any
 
 import click
 from click.shell_completion import CompletionItem
@@ -49,13 +49,10 @@ class DropboxPath(click.ParamType):
         param: click.Parameter | None,
         ctx: click.Context | None,
     ) -> str | None:
-
         if value is None:
             return value
-
         if not value.startswith("/"):
             value = "/" + value
-
         return value
 
     def shell_complete(
@@ -64,7 +61,6 @@ class DropboxPath(click.ParamType):
         param: click.Parameter | None,
         incomplete: str,
     ) -> list[CompletionItem]:
-
         from click.shell_completion import CompletionItem
         from ..utils import removeprefix
         from ..config import MaestralConfig
@@ -128,7 +124,6 @@ class ConfigKey(click.ParamType):
         param: click.Parameter | None,
         incomplete: str,
     ) -> list[CompletionItem]:
-
         from click.shell_completion import CompletionItem
         from ..config.main import KEY_SECTION_MAP as KEYS
 
@@ -155,14 +150,12 @@ class ConfigName(click.ParamType):
         param: click.Parameter | None,
         ctx: click.Context | None,
     ) -> str | None:
-
         if value is None:
             return value
 
         from ..config import validate_config_name, list_configs
 
         if not self.existing:
-
             # accept all valid config names
             try:
                 return validate_config_name(value)
@@ -170,7 +163,6 @@ class ConfigName(click.ParamType):
                 raise CliException("Configuration name may not contain any whitespace")
 
         else:
-
             # accept only existing config names
             if value in list_configs():
                 return value
@@ -186,7 +178,6 @@ class ConfigName(click.ParamType):
         param: click.Parameter | None,
         incomplete: str,
     ) -> list[CompletionItem]:
-
         from click.shell_completion import CompletionItem
         from ..config import list_configs
 
@@ -216,7 +207,6 @@ class OrderedGroup(click.Group):
     def format_commands(
         self, ctx: click.Context, formatter: click.HelpFormatter
     ) -> None:
-
         commands = []
 
         for name in self.commands:
@@ -236,7 +226,6 @@ class OrderedGroup(click.Group):
 
             # format sections individually
             for section, cmd_list in self.sections.items():
-
                 rows = []
 
                 for name, cmd in cmd_list:
@@ -258,5 +247,5 @@ class CliException(click.ClickException):
     message.
     """
 
-    def show(self, file: IO | None = None) -> None:
+    def show(self, file: Any = None) -> None:
         warn(self.format_message())
