@@ -1,15 +1,17 @@
-# -*- coding: utf-8 -*-
 """
 This module contains the default configuration and state values and functions to return
 existing config or state instances for a specified config_name.
 """
 
-import threading
-from typing import Dict
+from __future__ import annotations
 
-from maestral import __version__
-from .base import get_conf_path, get_data_path
-from .user import UserConfig, DefaultsType
+import threading
+
+from packaging.version import Version
+
+from .user import UserConfig, _DefaultsType
+from .. import __version__
+from ..utils.appdirs import get_conf_path, get_data_path
 
 
 CONFIG_DIR_NAME = "maestral"
@@ -19,30 +21,32 @@ CONFIG_DIR_NAME = "maestral"
 #  Defaults
 # =============================================================================
 
-DEFAULTS_CONFIG: DefaultsType = {
-    "main": {
-        "path": "",  # dropbox folder location
-        "excluded_items": [],  # files and folders excluded from sync
-    },
-    "account": {
-        "account_id": "",  # dropbox account id, must match the saved account key
+DEFAULTS_CONFIG: _DefaultsType = {
+    "auth": {
+        "account_id": "",  # dropbox account id
+        "keyring": "automatic",  # keychain backend to use for credential storage
+        "token_access_type": "offline",
     },
     "app": {
         "notification_level": 15,  # desktop notification level, default: FILECHANGE
         "log_level": 20,  # log level for journal and file, default: INFO
         "update_notification_interval": 60 * 60 * 24 * 7,  # default: weekly
-        "keyring": "automatic",  # keychain backend to use for credential storage
+        "bandwidth_limit_up": 0.0,  # upload limit in bytes / sec (0 = unlimited)
+        "bandwidth_limit_down": 0.0,  # download limit in bytes / sec (0 = unlimited)
+        "max_parallel_uploads": 6,  # max number of parallel downloads
+        "max_parallel_downloads": 6,  # max number of parallel downloads
     },
     "sync": {
-        "reindex_interval": 60 * 60 * 24 * 14,  # default: every fortnight
-        "max_cpu_percent": 20.0,  # max usage target per cpu core, default: 20%
+        "path": "",  # dropbox folder location
+        "excluded_items": [],  # files and folders excluded from sync
+        "max_cpu_percent": 20.0,  # max CPU usage target (100% = all cores busy)
         "keep_history": 60 * 60 * 24 * 7,  # default: one week
         "upload": True,  # if download sync is enabled
         "download": True,  # if upload sync is enabled
     },
 }
 
-DEFAULTS_STATE: DefaultsType = {
+DEFAULTS_STATE: _DefaultsType = {
     "account": {  # account state, periodically updated from dropbox servers
         "email": "",
         "display_name": "",
@@ -50,12 +54,13 @@ DEFAULTS_STATE: DefaultsType = {
         "type": "",
         "usage": "",
         "usage_type": "",  # private vs business
-        "token_access_type": "",  # will be updated on completed OAuth
+        "path_root_type": "user",  # the root folder type: team or user
+        "path_root_nsid": "",  # the namespace id of the root path
+        "home_path": "",  # the path of the user folder if not the root path
     },
     "app": {  # app state
         "updated_scripts_completed": __version__,
         "update_notification_last": 0.0,
-        "latest_release": "0.0.0",
     },
     "sync": {  # sync state, updated by monitor
         "cursor": "",  # remote cursor: represents last state synced from dropbox
@@ -63,8 +68,6 @@ DEFAULTS_STATE: DefaultsType = {
         "last_reindex": 0.0,  # time-stamp of full last reindexing
         "indexing_counter": 0,  # counter for indexing progress between restarts
         "did_finish_indexing": False,  # indicates completed indexing
-        "upload_errors": [],  # failed uploads to retry on next sync
-        "download_errors": [],  # failed downloads to retry on next sync
         "pending_uploads": [],  # incomplete uploads to retry on next sync
         "pending_downloads": [],  # incomplete downloads to retry on next sync
     },
@@ -80,12 +83,12 @@ for section_name, section_values in DEFAULTS_CONFIG.items():
 
 # IMPORTANT NOTES:
 # 1. If you want to *change* the default value of a current option, you need to
-#    do a MINOR update in config version, e.g. from 3.0.0 to 3.1.0
+#    do a MINOR update in config version, e.g. from 3.0 to 3.1
 # 2. If you want to *remove* options that are no longer needed in our codebase,
 #    or if you want to *rename* options, then you need to do a MAJOR update in
-#    version, e.g. from 3.0.0 to 4.0.0
+#    version, e.g. from 3.0 to 4.0
 # 3. You don't need to touch this value if you're just adding a new option
-CONF_VERSION = "16.0.0"
+CONF_VERSION = Version("20.0")
 
 
 # =============================================================================
@@ -96,21 +99,18 @@ CONF_VERSION = "16.0.0"
 def _get_conf(
     config_name: str,
     config_path: str,
-    defaults: DefaultsType,
-    registry: Dict[str, UserConfig],
-):
-
+    defaults: _DefaultsType,
+    registry: dict[str, UserConfig],
+) -> UserConfig:
     try:
         conf = registry[config_name]
     except KeyError:
-
         try:
             conf = UserConfig(
                 config_path,
                 defaults=defaults,
                 version=CONF_VERSION,
                 backup=True,
-                remove_obsolete=True,
             )
         except OSError:
             conf = UserConfig(
@@ -118,7 +118,6 @@ def _get_conf(
                 defaults=defaults,
                 version=CONF_VERSION,
                 backup=True,
-                remove_obsolete=True,
                 load=False,
             )
 
@@ -127,7 +126,7 @@ def _get_conf(
     return conf
 
 
-_config_instances: Dict[str, UserConfig] = {}
+_config_instances: dict[str, UserConfig] = {}
 _config_lock = threading.Lock()
 
 
@@ -147,7 +146,7 @@ def MaestralConfig(config_name: str) -> UserConfig:
         return _get_conf(config_name, config_path, DEFAULTS_CONFIG, _config_instances)
 
 
-_state_instances: Dict[str, UserConfig] = {}
+_state_instances: dict[str, UserConfig] = {}
 _state_lock = threading.Lock()
 
 
