@@ -4,9 +4,11 @@ be kept free of memory heavy imports.
 """
 
 # system imports
+import sys
 import platform
 import pathlib
 from enum import Enum
+from importlib_metadata import metadata, PackageNotFoundError
 from typing import ContextManager
 
 try:
@@ -19,11 +21,26 @@ except ImportError:
     from importlib.resources import path as resource_path
 
 
+FROZEN = getattr(sys, "frozen", False)
+
+for package in (
+    __package__,
+    "maestral",
+    "maestral-cocoa",
+    "maestral-qt",
+    "maestral-gui",
+):
+    try:
+        FROZEN = "Briefcase-Version" in metadata(package) or FROZEN
+    except PackageNotFoundError:
+        pass
+
 # app
 APP_NAME = "Maestral"
 BUNDLE_ID = "com.samschott.maestral"
 APP_ICON_PATH = resource_path("maestral.resources", "maestral.png").__enter__()
 ENV = {"PYTHONOPTIMIZE": "2", "LC_CTYPE": "UTF-8"}
+DEFAULT_CONFIG_NAME = "maestral"
 
 # sync
 OLD_REV_FILE = ".maestral"
