@@ -1,40 +1,47 @@
-# -*- coding: utf-8 -*-
 """
 This module defines functions to start and stop the sync daemon and retrieve proxy
 objects for a running daemon.
 """
 
+from __future__ import annotations
+
 # system imports
+import inspect
 import sys
 import os
 import time
 import signal
 import enum
-import subprocess
 import threading
 import fcntl
 import struct
-import warnings
 import argparse
-import ast
 import re
+import pickle
 from pprint import pformat
 from shlex import quote
-from typing import Optional, Any, Union, Tuple, Dict, Iterable, Type, TYPE_CHECKING
+from typing import Any, Iterable, ContextManager, TYPE_CHECKING
 from types import TracebackType
 
 # external imports
-import Pyro5  # type: ignore
-from Pyro5.errors import CommunicationError  # type: ignore
-from Pyro5.api import Daemon, Proxy, expose, oneway, register_dict_to_class  # type: ignore
-import sdnotify  # type: ignore
-from fasteners import InterProcessLock  # type: ignore
+import Pyro5
+from Pyro5.errors import CommunicationError
+from Pyro5.api import (
+    Daemon,
+    Proxy,
+    expose,
+    register_dict_to_class,
+    register_class_to_dict,
+)
+from Pyro5.serializers import serpent
+from fasteners import InterProcessLock
 
 # local imports
-from .errors import SYNC_ERRORS, GENERAL_ERRORS, MaestralApiError
 from .utils import exc_info_tuple
 from .utils.appdirs import get_runtime_path
+from .utils.integration import SystemdNotifier
 from .constants import IS_MACOS, ENV
+from . import core, models, exceptions
 
 
 if TYPE_CHECKING:
@@ -61,13 +68,10 @@ __all__ = [
 
 
 # systemd environment
-INVOCATION_ID = os.getenv("INVOCATION_ID")
 NOTIFY_SOCKET = os.getenv("NOTIFY_SOCKET")
-WATCHDOG_PID = os.getenv("WATCHDOG_PID")
+WATCHDOG_PID = int(os.getenv("WATCHDOG_PID", os.getpid()))
 WATCHDOG_USEC = os.getenv("WATCHDOG_USEC")
-IS_WATCHDOG = WATCHDOG_USEC and (
-    WATCHDOG_PID is None or int(WATCHDOG_PID) == os.getpid()
-)
+IS_WATCHDOG = WATCHDOG_USEC and WATCHDOG_PID == os.getpid()
 
 
 URI = "PYRO:maestral.{0}@{1}"
@@ -80,20 +84,16 @@ def freeze_support() -> None:
     This call will start the sync daemon if a matching command line arguments are
     detected and do nothing otherwise.
     """
-
     parser = argparse.ArgumentParser(add_help=False)
     parser.add_argument("-c")
     parsed_args, _ = parser.parse_known_args()
 
     if parsed_args.c:
-        template = r'.*start_maestral_daemon\("(?P<config_name>\S+)", start_sync=(?P<start_sync>\S+)\).*'
+        template = r'.*start_maestral_daemon\("(?P<config_name>\S+)"\).*'
         match = re.match(template, parsed_args.c)
 
         if match:
-            config_name = match["config_name"]
-            start_sync = ast.literal_eval(match["start_sync"])
-
-            start_maestral_daemon(config_name, start_sync)
+            start_maestral_daemon(match["config_name"])
             sys.exit()
 
 
@@ -117,79 +117,46 @@ class Start(enum.Enum):
 # ==== error serialization =============================================================
 
 
-def serpent_deserialize_api_error(class_name: str, d: dict) -> MaestralApiError:
+def check_signature(signature: str, obj: bytes) -> None:
+    pass
+
+
+def serialize_api_types(obj: Any) -> dict[str, Any]:
     """
-    Deserializes a MaestralApiError.
+    :param obj: Object to serialize.
+    :returns: Serialized object.
+    """
+    res = pickle.dumps(obj)
+    return {"__class__": type(obj).__name__, "object": res, "signature": ""}
+
+
+def deserialize_api_types(class_name: str, d: dict[str, Any]) -> Any:
+    """
+    Deserializes an API type. Allowed classes are defined in:
+        * :mod:`maestral.core`
+        * :mod:`maestral.model`
+        * :mod:`maestral.exceptions`
 
     :param class_name: Name of class to deserialize.
     :param d: Dictionary of serialized class.
-    :returns: Class instance.
+    :returns: Deserialized object.
     """
-
-    # Import maestral.errors for evaluation.
-    # This import needs to be absolute to reconstruct the Exception class. Note that the
-    # eval is safe here because ``serpent_deserialize_api_error`` is only registered for
-    # strings that match an error class name.
-
-    import maestral.errors  # noqa: F401
-
-    cls = eval(class_name)
-    err = cls(*d["args"])
-    for a_name, a_value in d["attributes"].items():
-        setattr(err, a_name, a_value)
-
-    return err
+    bytes_message = serpent.tobytes(d["object"])
+    check_signature(d["signature"], bytes_message)
+    return pickle.loads(bytes_message)
 
 
-for err_cls in (*SYNC_ERRORS, *GENERAL_ERRORS):
-    register_dict_to_class(
-        err_cls.__module__ + "." + err_cls.__name__, serpent_deserialize_api_error
-    )
+for module in core, models, exceptions:
+    for klass_name, klass in inspect.getmembers(module, inspect.isclass):
+        register_class_to_dict(klass, serialize_api_types)
+        register_dict_to_class(klass_name, deserialize_api_types)
 
 
 # ==== interprocess locking ============================================================
 
 
-def _get_lockdata() -> Tuple[bytes, str, int]:
-
-    try:
-        os.O_LARGEFILE
-    except AttributeError:
-        start_len = "ll"
-    else:
-        start_len = "qq"
-
-    if sys.platform.startswith(("netbsd", "freebsd", "openbsd")) or IS_MACOS:
-        if struct.calcsize("l") == 8:
-            off_t = "l"
-            pid_t = "i"
-        else:
-            off_t = "lxxxx"
-            pid_t = "l"
-
-        fmt = off_t + off_t + pid_t + "hh"
-        pid_index = 2
-        lockdata = struct.pack(fmt, 0, 0, 0, fcntl.F_WRLCK, 0)
-    # elif sys.platform.startswith('gnukfreebsd'):
-    #     fmt = 'qqihhi'
-    #     pid_index = 2
-    #     lockdata = struct.pack(fmt, 0, 0, 0, fcntl.F_WRLCK, 0, 0)
-    # elif sys.platform in ('hp-uxB', 'unixware7'):
-    #     fmt = 'hhlllii'
-    #     pid_index = 2
-    #     lockdata = struct.pack(fmt, fcntl.F_WRLCK, 0, 0, 0, 0, 0, 0)
-    elif sys.platform.startswith("linux"):
-        fmt = "hh" + start_len + "ih"
-        pid_index = 4
-        lockdata = struct.pack(fmt, fcntl.F_WRLCK, 0, 0, 0, 0, 0)
-    else:
-        raise RuntimeError(f"Unsupported platform {sys.platform}")
-
-    return lockdata, fmt, pid_index
-
-
 class Lock:
-    """A inter-process and inter-thread lock
+    """An inter-process and inter-thread lock
 
     This internally uses :class:`fasteners.InterProcessLock` but provides non-blocking
     acquire. It also guarantees thread-safety when using the :meth:`singleton` class
@@ -198,76 +165,51 @@ class Lock:
     :param path: Path of the lock file to use / create.
     """
 
-    _instances: Dict[str, "Lock"] = {}
+    _instances: dict[str, Lock] = {}
     _singleton_lock = threading.Lock()
 
     @classmethod
-    def singleton(cls, path: str) -> "Lock":
+    def singleton(cls, path: str) -> Lock:
         """
         Retrieve an existing lock object with a given 'name' or create a new one. Use
         this method for thread-safe locks.
 
         :param path: Path of the lock file to use / create.
         """
-
         with cls._singleton_lock:
-            try:
-                instance = cls._instances[path]
-            except KeyError:
-                instance = cls(path)
-                cls._instances[path] = instance
+            path = os.path.abspath(path)
 
-            return instance
+            if path not in cls._instances:
+                cls._instances[path] = cls(path)
+
+            return cls._instances[path]
 
     def __init__(self, path: str) -> None:
-
         self.path = path
-
-        self._internal_lock = threading.Semaphore()
+        self.pid = os.getpid()
         self._external_lock = InterProcessLock(self.path)
-
         self._lock = threading.RLock()
-        self._acquired = False  # tracks if we acquired the lock
 
     def acquire(self) -> bool:
         """
         Attempts to acquire the given lock.
 
-        :returns: Whether or not the acquisition succeeded.
+        :returns: Whether the acquisition succeeded.
         """
-
         with self._lock:
-            locked_internal = self._internal_lock.acquire(blocking=False)
-
-            if not locked_internal:
+            if self._external_lock.acquired:
                 return False
-
-            try:
-                locked_external = self._external_lock.acquire(blocking=False)
-            except Exception:
-                self._internal_lock.release()
-                raise
-            else:
-
-                if locked_external:
-                    self._acquired = True
-                    return True
-                else:
-                    self._internal_lock.release()
-                    return False
+            return self._external_lock.acquire(blocking=False)
 
     def release(self) -> None:
         """Release the previously acquired lock."""
         with self._lock:
-
-            if not self._acquired:
+            if not self._external_lock.acquired:
                 raise RuntimeError(
                     "Cannot release a lock, it was acquired by a different process"
                 )
 
             self._external_lock.release()
-            self._internal_lock.release()
-            self._acquired = False
 
     def locked(self) -> bool:
         """
@@ -281,7 +223,7 @@ class Lock:
                 self.release()
             return not gotten
 
-    def locking_pid(self) -> Optional[int]:
+    def locking_pid(self) -> int | None:
         """
         Returns the PID of the process which currently holds the lock or ``None``. This
         should work on macOS, OpenBSD and Linux but may fail on some platforms. Always
@@ -289,28 +231,30 @@ class Lock:
 
         :returns: The PID of the process which currently holds the lock or ``None``.
         """
-
         with self._lock:
-
-            if self._acquired:
-                return os.getpid()
+            if self._external_lock.acquired:
+                return self.pid
 
             try:
-                # Don't close again in case we are the locking process.
-                self._external_lock._do_open()
-                lockdata, fmt, pid_index = _get_lockdata()
-                lockdata = fcntl.fcntl(
-                    self._external_lock.lockfile, fcntl.F_GETLK, lockdata
-                )
-
-                lockdata_list = struct.unpack(fmt, lockdata)
-                pid = lockdata_list[pid_index]
-
-                if pid > 0:
-                    return pid
-
+                fh = open(self._external_lock.path, "a")
             except OSError:
-                pass
+                return None
+
+            if IS_MACOS:
+                fmt = "qqihh"
+                pid_index = 2
+                flock = struct.pack(fmt, 0, 0, 0, fcntl.F_WRLCK, 0)
+            else:
+                fmt = "hhqqih"
+                pid_index = 4
+                flock = struct.pack(fmt, fcntl.F_WRLCK, 0, 0, 0, 0, 0)
+
+            lockdata = fcntl.fcntl(fh.fileno(), fcntl.F_GETLK, flock)
+            lockdata_list = struct.unpack(fmt, lockdata)
+            pid = lockdata_list[pid_index]
+
+            if pid > 0:
+                return pid
 
             return None
 
@@ -318,9 +262,9 @@ class Lock:
 # ==== helpers for daemon management ===================================================
 
 
-def _send_term(pid: int) -> None:
+def _send_signal(pid: int, sig: int) -> None:
     try:
-        os.kill(pid, signal.SIGTERM)
+        os.kill(pid, sig)
     except ProcessLookupError:
         pass
 
@@ -328,7 +272,7 @@ def _send_term(pid: int) -> None:
 def maestral_lock(config_name: str) -> Lock:
     """
     Returns an inter-process and inter-thread lock for Maestral. This is a wrapper
-    around :class:`Lock` which fills out the appropriate lockfile path for he given
+    around :class:`Lock` which fills out the appropriate lockfile path for the given
     config name.
 
     :param config_name: The name of the Maestral configuration.
@@ -361,14 +305,13 @@ def lockpath_for_config(config_name: str) -> str:
     return get_runtime_path("maestral", f"{config_name}.lock")
 
 
-def get_maestral_pid(config_name: str) -> Optional[int]:
+def get_maestral_pid(config_name: str) -> int | None:
     """
     Returns the PID of the daemon if it is running, ``None`` otherwise.
 
     :param config_name: The name of the Maestral configuration.
     :returns: The daemon's PID.
     """
-
     return maestral_lock(config_name).locking_pid()
 
 
@@ -379,11 +322,10 @@ def is_running(config_name: str) -> bool:
     :param config_name: The name of the Maestral configuration.
     :returns: Whether the daemon is running.
     """
-
     return maestral_lock(config_name).locked()
 
 
-def wait_for_startup(config_name: str, timeout: float = 5) -> None:
+def wait_for_startup(config_name: str, timeout: float = 30) -> None:
     """
     Waits until we can communicate with the maestral daemon for ``config_name``.
 
@@ -392,7 +334,6 @@ def wait_for_startup(config_name: str, timeout: float = 5) -> None:
     :raises CommunicationError: if we cannot communicate with the daemon within the
         given timeout.
     """
-
     sock_name = sockpath_for_config(config_name)
     maestral_daemon = Proxy(URI.format(config_name, "./u:" + sock_name))
 
@@ -415,7 +356,7 @@ def wait_for_startup(config_name: str, timeout: float = 5) -> None:
 
 
 def start_maestral_daemon(
-    config_name: str = "maestral", log_to_stderr: bool = False, start_sync: bool = False
+    config_name: str = "maestral", log_to_stderr: bool = False
 ) -> None:
     """
     Starts the Maestral daemon with event loop in the current thread.
@@ -430,8 +371,6 @@ def start_maestral_daemon(
 
     :param config_name: The name of the Maestral configuration to use.
     :param log_to_stderr: If ``True``, write logs to stderr.
-    :param start_sync: If ``True``, start syncing once the daemon has started. If the
-        ``start_sync`` call fails, an error will be logged but not raised.
     :raises RuntimeError: if a daemon for the given ``config_name`` is already running.
     """
 
@@ -439,94 +378,92 @@ def start_maestral_daemon(
     from .main import Maestral
     from .logging import scoped_logger, setup_logging
 
-    setup_logging(config_name, log_to_stderr)
+    setup_logging(config_name, stderr=log_to_stderr)
     dlogger = scoped_logger(__name__, config_name)
+    sd_notifier = SystemdNotifier()
 
     dlogger.info("Starting daemon")
 
+    # ==== Process and thread management ===========================================
+
     if threading.current_thread() is not threading.main_thread():
         dlogger.error("Must run daemon in main thread")
-        raise RuntimeError("Must run daemon in main thread")
+        return
 
     dlogger.debug("Environment:\n%s", pformat(os.environ.copy()))
 
-    # acquire PID lock file
+    # Acquire PID lock file.
     lock = maestral_lock(config_name)
 
     if lock.acquire():
-        dlogger.debug("Acquired daemon lock: %s", lock.path)
+        dlogger.debug("Acquired daemon lock: %r", lock.path)
     else:
         dlogger.error("Could not acquire lock, daemon is already running")
-        raise RuntimeError("Daemon is already running")
+        return
 
-    # Nice ourselves to give other processes priority.
-    os.nice(10)
+    try:
+        # Nice ourselves to give other processes priority.
+        os.nice(10)
 
-    # Integrate with CFRunLoop in macOS.
-    if IS_MACOS:
+        # ==== System integration ======================================================
+        # Integrate with CFRunLoop in macOS.
+        event_loop_policy: asyncio.AbstractEventLoopPolicy
+        if IS_MACOS:
+            dlogger.debug("Integrating with CFEventLoop")
 
-        dlogger.debug("Integrating with CFEventLoop")
+            from rubicon.objc.eventloop import EventLoopPolicy
 
-        from rubicon.objc.eventloop import EventLoopPolicy  # type: ignore
+            event_loop_policy = EventLoopPolicy()
 
-        asyncio.set_event_loop_policy(EventLoopPolicy())
+        else:
+            event_loop_policy = asyncio.get_event_loop_policy()
 
-    # Get the default event loop.
-    loop = asyncio.get_event_loop()
+        # Get the default event loop.
+        loop = event_loop_policy.new_event_loop()
 
-    sd_notifier = sdnotify.SystemdNotifier()
+        # Notify systemd that we have started.
+        if NOTIFY_SOCKET:
+            dlogger.debug("Running as systemd notify service")
+            dlogger.debug("NOTIFY_SOCKET = %s", NOTIFY_SOCKET)
 
-    # Notify systemd that we have started.
-    if NOTIFY_SOCKET:
-        dlogger.debug("Running as systemd notify service")
-        dlogger.debug("NOTIFY_SOCKET = %s", NOTIFY_SOCKET)
         sd_notifier.notify("READY=1")
 
-    # Notify systemd periodically if alive.
-    if IS_WATCHDOG and WATCHDOG_USEC:
+        # Notify systemd periodically if alive.
+        if IS_WATCHDOG and WATCHDOG_USEC:
 
-        async def periodic_watchdog() -> None:
+            async def periodic_watchdog() -> None:
+                if WATCHDOG_USEC:
+                    sleep = int(WATCHDOG_USEC)
+                    while True:
+                        sd_notifier.notify("WATCHDOG=1")
+                        await asyncio.sleep(sleep / (2 * 10**6))
 
-            if WATCHDOG_USEC:
+            dlogger.debug("Running as systemd watchdog service")
+            dlogger.debug("WATCHDOG_USEC = %s", WATCHDOG_USEC)
+            dlogger.debug("WATCHDOG_PID = %s", WATCHDOG_PID)
+            loop.create_task(periodic_watchdog())
 
-                sleep = int(WATCHDOG_USEC)
-                while True:
-                    sd_notifier.notify("WATCHDOG=1")
-                    await asyncio.sleep(sleep / (2 * 10 ** 6))
+        # ==== Run Maestral as Pyro server =============================================
+        # Get socket for config name.
+        sockpath = sockpath_for_config(config_name)
+        dlogger.debug("Socket path: %r", sockpath)
 
-        dlogger.debug("Running as systemd watchdog service")
-        dlogger.debug("WATCHDOG_USEC = %s", WATCHDOG_USEC)
-        dlogger.debug("WATCHDOG_PID = %s", WATCHDOG_PID)
-        loop.create_task(periodic_watchdog())
+        # Clean up old socket.
+        try:
+            os.remove(sockpath)
+        except (FileNotFoundError, NotADirectoryError):
+            pass
 
-    # Get socket for config name.
-    sockpath = sockpath_for_config(config_name)
-    dlogger.debug(f"Socket path: '{sockpath}'")
+        # Expose maestral as Pyro server.
+        dlogger.debug("Creating Pyro daemon")
 
-    # Clean up old socket.
-    try:
-        os.remove(sockpath)
-    except FileNotFoundError:
-        pass
-
-    # Expose maestral as Pyro server. Convert management
-    # methods to one way calls so that they don't block.
-
-    dlogger.debug("Creating Pyro daemon")
-
-    ExposedMaestral = expose(Maestral)
-
-    ExposedMaestral.start_sync = oneway(ExposedMaestral.start_sync)
-    ExposedMaestral.stop_sync = oneway(ExposedMaestral.stop_sync)
-    ExposedMaestral.shutdown_daemon = oneway(ExposedMaestral.shutdown_daemon)
-
-    maestral_daemon = ExposedMaestral(config_name, log_to_stderr=log_to_stderr)
-
-    if start_sync:
-        dlogger.debug("Starting sync")
-        maestral_daemon.start_sync()
-
-    try:
+        shutdown_future = loop.create_future()
+        maestral_daemon = expose(Maestral)(
+            config_name,
+            log_to_stderr=log_to_stderr,
+            event_loop=loop,
+            shutdown_future=shutdown_future,
+        )
 
         dlogger.debug("Starting event loop")
 
@@ -546,7 +483,7 @@ def start_maestral_daemon(
             for s in signals:
                 loop.add_signal_handler(s, maestral_daemon.shutdown_daemon)
 
-            loop.run_until_complete(maestral_daemon.shutdown_complete)
+            loop.run_until_complete(shutdown_future)
 
             for socket in daemon.sockets:
                 loop.remove_reader(socket.fileno())
@@ -557,16 +494,14 @@ def start_maestral_daemon(
     except Exception as exc:
         dlogger.error(exc.args[0], exc_info=True)
     finally:
+        # Notify systemd that we are shutting down.
+        sd_notifier.notify("STOPPING=1")
 
-        if NOTIFY_SOCKET:
-            # Notify systemd that we are shutting down.
-            sd_notifier.notify("STOPPING=1")
+        lock.release()
 
 
 def start_maestral_daemon_process(
-    config_name: str = "maestral",
-    start_sync: bool = False,
-    timeout: int = 5,
+    config_name: str = "maestral", timeout: float = 30
 ) -> Start:
     """
     Starts the Maestral daemon in a new process by calling :func:`start_maestral_daemon`.
@@ -579,57 +514,44 @@ def start_maestral_daemon_process(
     the environment variables defined in :const:`constants.ENV`.
 
     :param config_name: The name of the Maestral configuration to use.
-    :param start_sync: If ``True``, start syncing once the daemon has started.
     :param timeout: Time in sec to wait for daemon to start.
     :returns: :attr:`Start.Ok` if successful, :attr:`Start.AlreadyRunning` if the daemon
         was already running or :attr:`Start.Failed` if startup failed. It is possible
         that :attr:`Start.Ok` may be returned instead of :attr:`Start.AlreadyRunning`
         in case of a race but the daemon is nevertheless started only once.
     """
-
     if is_running(config_name):
         return Start.AlreadyRunning
 
     # Protect against injection.
     cc = quote(config_name).strip("'")
-    start_sync = bool(start_sync)
 
-    script = (
-        f"import maestral.daemon; "
-        f'maestral.daemon.start_maestral_daemon("{cc}", start_sync={start_sync})'
-    )
+    script = f'import maestral.daemon; maestral.daemon.start_maestral_daemon("{cc}")'
 
-    cmd = [sys.executable, "-c", script]
+    env = os.environ.copy()
+    env.update(ENV)
 
-    subprocess_env = os.environ.copy()
-    subprocess_env.update(ENV)
-
-    process = subprocess.Popen(
-        cmd,
-        start_new_session=True,
-        stdin=subprocess.DEVNULL,
-        stdout=subprocess.DEVNULL,
-        stderr=subprocess.DEVNULL,
-        env=subprocess_env,
-    )
+    pid = os.spawnve(os.P_NOWAIT, sys.executable, [sys.executable, "-c", script], env)
 
     try:
         wait_for_startup(config_name, timeout)
     except Exception as exc:
         from .logging import scoped_logger, setup_logging
 
-        setup_logging(config_name, log_to_stderr=False)
+        setup_logging(config_name, stderr=False)
         clogger = scoped_logger(__name__, config_name)
 
         clogger.error("Could not communicate with daemon", exc_info=exc_info_tuple(exc))
 
-        # Let's check what the daemon has been doing.
-        returncode = process.poll()
-        if returncode is None:
+        # Let's check if the daemon is running
+        try:
+            os.kill(pid, 0)
             clogger.error("Daemon is running but not responsive, killing now")
-            process.terminate()
+        except OSError:
+            clogger.error("Daemon quit unexpectedly")
         else:
-            clogger.error("Daemon stopped with return code %s", returncode)
+            os.kill(pid, signal.SIGTERM)
+
         return Start.Failed
     else:
         return Start.Ok
@@ -650,42 +572,30 @@ def stop_maestral_daemon_process(
         :attr:`Stop.NotRunning` if the daemon was not running and :attr:`Stop.Failed`
         if killing the process failed because we could not retrieve its PID.
     """
-
     if not is_running(config_name):
         return Stop.NotRunning
 
     pid = get_maestral_pid(config_name)
 
-    try:
-        with MaestralProxy(config_name) as m:
-            m.shutdown_daemon()
-    except CommunicationError:
-        if pid:
-            _send_term(pid)
-    finally:
-        while timeout > 0:
-            if not is_running(config_name):
-                return Stop.Ok
-            else:
-                time.sleep(0.2)
-                timeout -= 0.2
+    if not pid:
+        # Cannot send SIGTERM to process if we don't know its pid.
+        return Stop.Failed
 
-        # Send SIGTERM after timeout and delete PID file.
-        if pid:
-            _send_term(pid)
-
-        time.sleep(1)
+    _send_signal(pid, signal.SIGTERM)
 
+    while timeout > 0:
         if not is_running(config_name):
             return Stop.Ok
-        elif pid:
-            os.kill(pid, signal.SIGKILL)
-            return Stop.Killed
         else:
-            return Stop.Failed
+            time.sleep(0.2)
+            timeout -= 0.2
+
+    # Kill.
+    _send_signal(pid, signal.SIGKILL)
+    return Stop.Killed
 
 
-class MaestralProxy:
+class MaestralProxy(ContextManager["MaestralProxy"]):
     """A Proxy to the Maestral daemon
 
     All methods and properties of Maestral's public API are accessible and calls /
@@ -696,34 +606,34 @@ class MaestralProxy:
 
         Use MaestralProxy as a context manager:
 
+        >>> import src.maestral.cli.cli_info
         >>> with MaestralProxy() as m:
-        ...     print(m.status)
+        ...     print(src.maestral.cli.cli_info.status)
 
         Use MaestralProxy directly:
 
+        >>> import src.maestral.cli.cli_info
         >>> m = MaestralProxy()
-        >>> print(m.status)
+        >>> print(src.maestral.cli.cli_info.status)
         >>> m._disconnect()
 
     :ivar _is_fallback: Whether we are using an actual Maestral instance as fallback
         instead of a Proxy.
 
     :param config_name: The name of the Maestral configuration to use.
-    :param fallback: If ``True``, a new instance of Maestral will created in the current
-        process when the daemon is not running.
+    :param fallback: If ``True``, a new instance of Maestral will be created in the
+        current process when the daemon is not running.
     :raises CommunicationError: if the daemon is running but cannot be reached or if the
         daemon is not running and ``fallback`` is ``False``.
     """
 
-    _m: Union["Maestral", Proxy]
+    _m: Maestral | Proxy
 
     def __init__(self, config_name: str = "maestral", fallback: bool = False) -> None:
-
         self._config_name = config_name
         self._is_fallback = False
 
         if is_running(config_name):
-
             sock_name = sockpath_for_config(config_name)
 
             # print remote tracebacks locally
@@ -736,15 +646,12 @@ class MaestralProxy:
                 self._m._pyroRelease()
                 raise
 
-        else:
-            # If daemon is not running, fall back to new Maestral instance
-            # or raise a CommunicationError if fallback not allowed.
-            if fallback:
-                from .main import Maestral
+        elif fallback:
+            from .main import Maestral
 
-                self._m = Maestral(config_name)
-            else:
-                raise CommunicationError(f"Could not get proxy for '{config_name}'")
+            self._m = Maestral(config_name)
+        else:
+            raise CommunicationError(f"Could not get proxy for '{config_name}'")
 
         self._is_fallback = not isinstance(self._m, Proxy)
 
@@ -752,11 +659,14 @@ class MaestralProxy:
         if isinstance(self._m, Proxy):
             self._m._pyroRelease()
 
-    def __enter__(self) -> "MaestralProxy":
+    def __enter__(self) -> MaestralProxy:
         return self
 
     def __exit__(
-        self, exc_type: Type[Exception], exc_value: Exception, tb: TracebackType
+        self,
+        exc_type: type[BaseException] | None,
+        exc_value: BaseException | None,
+        tb: TracebackType | None,
     ) -> None:
         self._disconnect()
         del self._m
@@ -769,7 +679,7 @@ class MaestralProxy:
         else:
             return self._m.__getattribute__(item)
 
-    def __setattr__(self, key, value) -> None:
+    def __setattr__(self, key: str, value: Any) -> None:
         if key.startswith("_"):
             super().__setattr__(key, value)
         else:
@@ -786,16 +696,3 @@ class MaestralProxy:
             f"<{self.__class__.__name__}(config={self._config_name!r}, "
             f"is_fallback={self._is_fallback})>"
         )
-
-
-def get_maestral_proxy(
-    config_name: str = "maestral", fallback: bool = False
-) -> Union["Maestral", Proxy]:
-
-    warnings.warn(
-        "'get_maestral_proxy' is deprecated, please use 'MaestralProxy' instead",
-        DeprecationWarning,
-    )
-
-    m = MaestralProxy(config_name, fallback=fallback)
-    return m._m
