@@ -77,7 +77,16 @@ from .utils.path import (
 )
 from .utils.appdirs import get_cache_path, get_data_path
 from .database.core import Database
-from .constants import IDLE, PAUSED, CONNECTING, GITHUB_RELEASES_API, FileStatus
+from .constants import (
+    IS_LINUX,
+    IS_MACOS,
+    IDLE,
+    PAUSED,
+    CONNECTING,
+    DEFAULT_CONFIG_NAME,
+    GITHUB_RELEASES_API,
+    FileStatus,
+)
 
 
 __all__ = ["Maestral"]
@@ -152,11 +161,14 @@ class Maestral:
 
     def __init__(
         self,
-        config_name: str = "maestral",
+        config_name: str = DEFAULT_CONFIG_NAME,
         log_to_stderr: bool = False,
         event_loop: AbstractEventLoop | None = None,
         shutdown_future: Future[bool] | None = None,
     ) -> None:
+        # Check system compatibility.
+        self._check_system_compatibility()
+
         self._loop = event_loop
         self._config_name = validate_config_name(config_name)
         self._conf = MaestralConfig(self.config_name)
@@ -197,6 +209,14 @@ class Maestral:
 
         self.shutdown_future = shutdown_future
 
+    @staticmethod
+    def _check_system_compatibility() -> None:
+        if os.stat not in os.supports_follow_symlinks:
+            raise RuntimeError("Maestral requires lstat support")
+
+        if not (IS_MACOS or IS_LINUX):
+            raise RuntimeError("Only macOS and Linux are supported")
+
     def _setup_logging_external(self) -> None:
         """
         Sets up logging to external channels:
@@ -641,17 +661,23 @@ class Maestral:
         local_path = osp.realpath(local_path)
 
         try:
-            dbx_path = self.sync.to_dbx_path(local_path)
-            dbx_path_lower = self.sync.to_dbx_path_lower(local_path)
+            dbx_path_cased = self.sync.to_dbx_path(local_path)
         except ValueError:
             return FileStatus.Unwatched.value
 
-        node = self.sync.activity.get_node(dbx_path)
+        # Find any sync activity for the local path.
+        node = self.sync.activity.get_node(dbx_path_cased)
+
         if not node:
+            # Always return synced for the root folder in the absense of sync activity.
+            if dbx_path_cased == "/":
+                return FileStatus.Synced.value
+
             # Check if the path is in our index. If yes, it is fully synced, otherwise
             # it is unwatched.
-            if dbx_path_lower == "/" or self.sync.get_local_rev(dbx_path_lower):
+            if self.sync.get_index_entry_for_local_path(local_path):
                 return FileStatus.Synced.value
+
             return FileStatus.Unwatched.value
 
         # Return effective status of item and its children. Syncing items take
@@ -995,7 +1021,10 @@ class Maestral:
         """
         self._check_linked()
         self._logger.info(f"Restoring '{dbx_path} to {rev}'")
-        return self.client.restore(dbx_path, rev)
+        res = self.client.restore(dbx_path, rev)
+        self._logger.info(f"Restored '{dbx_path} to {rev}'")
+        self._logger.info(IDLE)
+        return res
 
     def _delete_old_profile_pics(self) -> None:
         for file in os.listdir(get_cache_path("maestral")):
@@ -1461,6 +1490,8 @@ class Maestral:
         if self.shutdown_future and self._loop and self._loop.is_running():
             self._loop.call_soon_threadsafe(self.shutdown_future.set_result, True)
 
+        self._logger.info("Shutting down")
+
     # ==== Verifiers ===================================================================
 
     def _check_linked(self) -> None:
