@@ -16,6 +16,8 @@ from typing import TYPE_CHECKING
 # external imports
 from watchdog.events import (
     FileSystemEvent,
+    FileMovedEvent,
+    DirMovedEvent,
     EVENT_TYPE_CREATED,
     EVENT_TYPE_DELETED,
     EVENT_TYPE_MOVED,
@@ -45,6 +47,19 @@ __all__ = [
 ]
 
 
+def get_dest_path(event: FileSystemEvent) -> str:
+    """
+    Returns the dest_path of a file system event if present (moved events only)
+    otherwise returns the src_path (which is also the "destination").
+
+    :param event: Watchdog file system event.
+    :returns: Destination path for moved event, source path otherwise.
+    """
+    if isinstance(event, (FileMovedEvent, DirMovedEvent)):
+        return event.dest_path
+    return event.src_path
+
+
 class SyncDirection(enum.Enum):
     """Enumeration of sync directions"""
 
@@ -61,6 +76,7 @@ class SyncStatus(enum.Enum):
     Failed = "failed"
     Skipped = "skipped"
     Aborted = "aborted"
+    Conflict = "conflict"
 
 
 class ItemType(enum.Enum):
@@ -289,13 +305,8 @@ class SyncEvent(Model):
             dbx_id = None
             change_dbid = None
 
-            local_rev = sync_engine.get_local_rev(md.path_lower)
-            if local_rev == "folder":
-                item_type = ItemType.Folder
-            elif local_rev is not None:
-                item_type = ItemType.File
-            else:
-                item_type = ItemType.Unknown
+            entry = sync_engine.get_index_entry(md.path_lower)
+            item_type = ItemType.Unknown if entry is None else entry.item_type
 
         elif isinstance(md, FolderMetadata):
             # there is currently no API call to determine who added a folder
@@ -370,7 +381,7 @@ class SyncEvent(Model):
             # This is useful for testing.
             change_dbid = ""
 
-        to_path = getattr(event, "dest_path", event.src_path)
+        to_path = get_dest_path(event)
         from_path = None
 
         if event.event_type == EVENT_TYPE_CREATED:
@@ -421,6 +432,21 @@ class SyncEvent(Model):
         dbx_path_from = sync_engine.to_dbx_path(from_path) if from_path else None
         dbx_path_from_lower = normalize(dbx_path_from) if dbx_path_from else None
 
+        # For file changes, update the change type based on our index. Local file system
+        # events of created vs modified can be misleading for some safe save mechanisms.
+        # See https://github.com/samschott/maestral/issues/850.
+        if item_type is ItemType.File and change_type in {
+            ChangeType.Added,
+            ChangeType.Modified,
+        }:
+            entry = sync_engine.get_index_entry(dbx_path_lower)
+            if entry is None:
+                # Item is new to us.
+                change_type = ChangeType.Added
+            elif entry.item_type is ItemType.File:
+                # Item already existed.
+                change_type = ChangeType.Modified
+
         return cls(
             direction=SyncDirection.Up,
             item_type=item_type,
@@ -454,7 +480,7 @@ class IndexEntry(Model):
     Corresponds to the path_lower field of Dropbox metadata.
     """
 
-    dbx_path_cased = NonNullColumn(SqlPath())
+    dbx_path_cased = NonNullColumn(SqlPath(), index=True)
     """
     Dropbox path of the item, correctly cased. Corresponds to the path_display field of
     Dropbox metadata.
