@@ -30,31 +30,41 @@ _AnyPath = Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]
 # ==== path relationships ==============================================================
 
 
-def is_child(path: str, parent: str) -> bool:
+def is_child(path: str, parent: str, case_sensitive: bool = True) -> bool:
     """
     Checks if ``path`` semantically is inside ``parent``. Neither path needs to
     refer to an actual item on the drive. This function is case-sensitive.
 
     :param path: Item path.
     :param parent: Parent path.
+    :param case_sensitive: Whether to do case-sensitive checks.
     :returns: Whether ``path`` semantically lies inside ``parent``.
     """
+    if not case_sensitive:
+        path = normalize(path)
+        parent = normalize(parent)
+
     parent = parent.rstrip(osp.sep) + osp.sep
     path = path.rstrip(osp.sep)
 
     return path.startswith(parent)
 
 
-def is_equal_or_child(path: str, parent: str) -> bool:
+def is_equal_or_child(path: str, parent: str, case_sensitive: bool = True) -> bool:
     """
     Checks if ``path`` semantically is inside ``parent`` or equals ``parent``. Neither
     path needs to refer to an actual item on the drive. This function is case-sensitive.
 
     :param path: Item path.
     :param parent: Parent path.
+    :param case_sensitive: Whether to do case-sensitive checks.
     :returns: ``True`` if ``path`` semantically lies inside ``parent`` or
         ``path == parent``.
     """
+    if not case_sensitive:
+        path = normalize(path)
+        parent = normalize(parent)
+
     return is_child(path, parent) or path == parent
 
 
@@ -219,6 +229,10 @@ def to_existing_unnormalized_path(
     This is similar to :func:`get_existing_equivalent_paths` but returns only the first
     candidate or raises a :class:`FileNotFoundError` if no candidates can be found.
 
+    If the file system is not case-sensitive but case-preserving, this function
+    effectively returns the "displayed" version of a path, as used for example in file
+    managers.
+
     On macOS, we use fcntl F_GETPATH for a more efficient implementation.
 
     :param path: Original path relative to ``root``.
@@ -265,10 +279,11 @@ def normalized_path_exists(path: str, root: str = osp.sep) -> bool:
     return False
 
 
-def generate_cc_name(path: str, suffix: str = "conflicting copy") -> str:
+def generate_cc_name(path: str, suffix: str) -> str:
     """
     Generates a path for a conflicting copy of ``path``. The file name is created by
-    inserting the given ``suffix`` between the filename and the extension. For instance:
+    inserting the given ``suffix`` between the filename and the extension. For example,
+    for ``suffix = "conflicting copy"``:
 
         "my_file.txt" -> "my_file (conflicting copy).txt"
 
@@ -278,7 +293,7 @@ def generate_cc_name(path: str, suffix: str = "conflicting copy") -> str:
         "my_file.txt" -> "my_file (conflicting copy 1).txt"
 
     :param path: Original path name.
-    :param suffix: Suffix to use. Defaults to "conflicting copy".
+    :param suffix: Suffix to use.
     :returns: New path.
     """
     dirname, basename = osp.split(path)
@@ -313,7 +328,9 @@ def delete(
     """
     err: Optional[OSError] = None
 
-    if force_case_sensitive and path != to_existing_unnormalized_path(path):
+    if force_case_sensitive and not equal_but_for_unicode_norm(
+        path, to_existing_unnormalized_path(path)
+    ):
         err = FileNotFoundError(f"No such file '{path}'")
         if raise_error:
             raise err
@@ -345,6 +362,8 @@ def move(
     the destination path no longer exist, this function does nothing. Any other
     exceptions are either raised or returned if ``raise_error`` is False.
 
+    Uses ``os.rename`` internally.
+
     :param src_path: Path of item to move.
     :param dest_path: Destination path. Any existing file at this path will be replaced
         by the move. Any existing **empty** folder will be replaced if the source is
@@ -365,7 +384,7 @@ def move(
             pass
 
     try:
-        shutil.move(src_path, dest_path)
+        os.rename(src_path, dest_path)
     except FileNotFoundError:
         # do nothing if source or dest path no longer exist
         pass
@@ -375,7 +394,10 @@ def move(
         if orig_mode:
             # reapply dest permissions
             try:
-                os.chmod(dest_path, orig_mode, follow_symlinks=False)
+                if os.chmod in os.supports_follow_symlinks:
+                    os.chmod(dest_path, orig_mode, follow_symlinks=False)
+                else:
+                    os.chmod(dest_path, orig_mode)
             except OSError:
                 pass
 
@@ -545,6 +567,10 @@ def getsize(path: _AnyPath) -> int:
     return os.lstat(path).st_size
 
 
+def equal_but_for_unicode_norm(s0: str, s1: str) -> bool:
+    return normalize_unicode(s0) == normalize_unicode(s1)
+
+
 def get_symlink_target(local_path: str) -> Optional[str]:
     """
     Returns the symlink target of a file.
